/*
Copyright (c) 2011 Cimaron Shanahan

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


cWebGLRenderingContext = (function() {

	function Initializer() {
		//public:
		this.canvas = null;
		this.driver = null;
		this.attr = null;
		this._context = null;
		this._readyFunc = null;
		this._initialized = false;

		this._state = {};
		this.errors = [];
	}

	var cWebGLRenderingContext = jClass('cWebGLRenderingContext', Initializer);

	cWebGLRenderingContext.Enumerate({

	 /* ClearBufferMask */
		DEPTH_BUFFER_BIT               : 0x00000100,
		STENCIL_BUFFER_BIT             : 0x00000400,
		COLOR_BUFFER_BIT               : 0x00004000,
		
		/* BeginMode */
		POINTS                         : 0x0000,
		LINES                          : 0x0001,
		LINE_LOOP                      : 0x0002,
		LINE_STRIP                     : 0x0003,
		TRIANGLES                      : 0x0004,
		TRIANGLE_STRIP                 : 0x0005,
		TRIANGLE_FAN                   : 0x0006,

		/* AlphaFunction (not supported in ES20) */
		/*      NEVER */
		/*      LESS */
		/*      EQUAL */
		/*      LEQUAL */
		/*      GREATER */
		/*      NOTEQUAL */
		/*      GEQUAL */
		/*      ALWAYS */
		
		/* BlendingFactorDest */
		ZERO                           : 0,
		ONE                            : 1,
		SRC_COLOR                      : 0x0300,
		ONE_MINUS_SRC_COLOR            : 0x0301,
		SRC_ALPHA                      : 0x0302,
		ONE_MINUS_SRC_ALPHA            : 0x0303,
		DST_ALPHA                      : 0x0304,
		ONE_MINUS_DST_ALPHA            : 0x0305,
		
		/* BlendingFactorSrc */
		/*      ZERO */
		/*      ONE */
		DST_COLOR                      : 0x0306,
		ONE_MINUS_DST_COLOR            : 0x0307,
		SRC_ALPHA_SATURATE             : 0x0308,
		/*      SRC_ALPHA */
		/*      ONE_MINUS_SRC_ALPHA */
		/*      DST_ALPHA */
		/*      ONE_MINUS_DST_ALPHA */
		
		/* BlendEquationSeparate */
		FUNC_ADD                       : 0x8006,
		BLEND_EQUATION                 : 0x8009,
		BLEND_EQUATION_RGB             : 0x8009,   /* same as BLEND_EQUATION */
		BLEND_EQUATION_ALPHA           : 0x883D,
		
		/* BlendSubtract */
		FUNC_SUBTRACT                  : 0x800A,
		FUNC_REVERSE_SUBTRACT          : 0x800B,
		
		/* Separate Blend Functions */
		BLEND_DST_RGB                  : 0x80C8,
		BLEND_SRC_RGB                  : 0x80C9,
		BLEND_DST_ALPHA                : 0x80CA,
		BLEND_SRC_ALPHA                : 0x80CB,
		CONSTANT_COLOR                 : 0x8001,
		ONE_MINUS_CONSTANT_COLOR       : 0x8002,
		CONSTANT_ALPHA                 : 0x8003,
		ONE_MINUS_CONSTANT_ALPHA       : 0x8004,
		BLEND_COLOR                    : 0x8005,
		
		/* Buffer Objects */
		ARRAY_BUFFER                   : 0x8892,
		ELEMENT_ARRAY_BUFFER           : 0x8893,
		ARRAY_BUFFER_BINDING           : 0x8894,
		ELEMENT_ARRAY_BUFFER_BINDING   : 0x8895,
		
		STREAM_DRAW                    : 0x88E0,
		STATIC_DRAW                    : 0x88E4,
		DYNAMIC_DRAW                   : 0x88E8,
		
		BUFFER_SIZE                    : 0x8764,
		BUFFER_USAGE                   : 0x8765,
		
		CURRENT_VERTEX_ATTRIB          : 0x8626,
		
		/* CullFaceMode */
		FRONT                          : 0x0404,
		BACK                           : 0x0405,
		FRONT_AND_BACK                 : 0x0408,
		
		/* DepthFunction */
		/*      NEVER */
		/*      LESS */
		/*      EQUAL */
		/*      LEQUAL */
		/*      GREATER */
		/*      NOTEQUAL */
		/*      GEQUAL */
		/*      ALWAYS */
		
		/* EnableCap */
		/* TEXTURE_2D */
		CULL_FACE                      : 0x0B44,
		BLEND                          : 0x0BE2,
		DITHER                         : 0x0BD0,
		STENCIL_TEST                   : 0x0B90,
		DEPTH_TEST                     : 0x0B71,
		SCISSOR_TEST                   : 0x0C11,
		POLYGON_OFFSET_FILL            : 0x8037,
		SAMPLE_ALPHA_TO_COVERAGE       : 0x809E,
		SAMPLE_COVERAGE                : 0x80A0,
		
		/* ErrorCode */
		NO_ERROR                       : 0,
		INVALID_ENUM                   : 0x0500,
		INVALID_VALUE                  : 0x0501,
		INVALID_OPERATION              : 0x0502,
		OUT_OF_MEMORY                  : 0x0505,
		
		/* FrontFaceDirection */
		CW                             : 0x0900,
		CCW                            : 0x0901,
		
		/* GetPName */
		LINE_WIDTH                     : 0x0B21,
		ALIASED_POINT_SIZE_RANGE       : 0x846D,
		ALIASED_LINE_WIDTH_RANGE       : 0x846E,
		CULL_FACE_MODE                 : 0x0B45,
		FRONT_FACE                     : 0x0B46,
		DEPTH_RANGE                    : 0x0B70,
		DEPTH_WRITEMASK                : 0x0B72,
		DEPTH_CLEAR_VALUE              : 0x0B73,
		DEPTH_FUNC                     : 0x0B74,
		STENCIL_CLEAR_VALUE            : 0x0B91,
		STENCIL_FUNC                   : 0x0B92,
		STENCIL_FAIL                   : 0x0B94,
		STENCIL_PASS_DEPTH_FAIL        : 0x0B95,
		STENCIL_PASS_DEPTH_PASS        : 0x0B96,
		STENCIL_REF                    : 0x0B97,
		STENCIL_VALUE_MASK             : 0x0B93,
		STENCIL_WRITEMASK              : 0x0B98,
		STENCIL_BACK_FUNC              : 0x8800,
		STENCIL_BACK_FAIL              : 0x8801,
		STENCIL_BACK_PASS_DEPTH_FAIL   : 0x8802,
		STENCIL_BACK_PASS_DEPTH_PASS   : 0x8803,
		STENCIL_BACK_REF               : 0x8CA3,
		STENCIL_BACK_VALUE_MASK        : 0x8CA4,
		STENCIL_BACK_WRITEMASK         : 0x8CA5,
		VIEWPORT                       : 0x0BA2,
		SCISSOR_BOX                    : 0x0C10,
		/*      SCISSOR_TEST */
		COLOR_CLEAR_VALUE              : 0x0C22,
		COLOR_WRITEMASK                : 0x0C23,
		UNPACK_ALIGNMENT               : 0x0CF5,
		PACK_ALIGNMENT                 : 0x0D05,
		MAX_TEXTURE_SIZE               : 0x0D33,
		MAX_VIEWPORT_DIMS              : 0x0D3A,
		SUBPIXEL_BITS                  : 0x0D50,
		RED_BITS                       : 0x0D52,
		GREEN_BITS                     : 0x0D53,
		BLUE_BITS                      : 0x0D54,
		ALPHA_BITS                     : 0x0D55,
		DEPTH_BITS                     : 0x0D56,
		STENCIL_BITS                   : 0x0D57,
		POLYGON_OFFSET_UNITS           : 0x2A00,
		/*      POLYGON_OFFSET_FILL */
		POLYGON_OFFSET_FACTOR          : 0x8038,
		TEXTURE_BINDING_2D             : 0x8069,
		SAMPLE_BUFFERS                 : 0x80A8,
		SAMPLES                        : 0x80A9,
		SAMPLE_COVERAGE_VALUE          : 0x80AA,
		SAMPLE_COVERAGE_INVERT         : 0x80AB,
		
		/* GetTextureParameter */
		/*      TEXTURE_MAG_FILTER */
		/*      TEXTURE_MIN_FILTER */
		/*      TEXTURE_WRAP_S */
		/*      TEXTURE_WRAP_T */
		
		NUM_COMPRESSED_TEXTURE_FORMATS : 0x86A2,
		COMPRESSED_TEXTURE_FORMATS     : 0x86A3,
		
		/* HintMode */
		DONT_CARE                      : 0x1100,
		FASTEST                        : 0x1101,
		NICEST                         : 0x1102,
		
		/* HintTarget */
		GENERATE_MIPMAP_HINT            : 0x8192,
		
		/* DataType */
		BYTE                           : 0x1400,
		UNSIGNED_BYTE                  : 0x1401,
		SHORT                          : 0x1402,
		UNSIGNED_SHORT                 : 0x1403,
		INT                            : 0x1404,
		UNSIGNED_INT                   : 0x1405,
		FLOAT                          : 0x1406,
		
		/* PixelFormat */
		DEPTH_COMPONENT                : 0x1902,
		ALPHA                          : 0x1906,
		RGB                            : 0x1907,
		RGBA                           : 0x1908,
		LUMINANCE                      : 0x1909,
		LUMINANCE_ALPHA                : 0x190A,
		
		/* PixelType */
		/*      UNSIGNED_BYTE */
		UNSIGNED_SHORT_4_4_4_4         : 0x8033,
		UNSIGNED_SHORT_5_5_5_1         : 0x8034,
		UNSIGNED_SHORT_5_6_5           : 0x8363,
	
		/* Shaders */
		FRAGMENT_SHADER                  : 0x8B30,
		VERTEX_SHADER                    : 0x8B31,
		MAX_VERTEX_ATTRIBS               : 0x8869,
		MAX_VERTEX_UNIFORM_VECTORS       : 0x8DFB,
		MAX_VARYING_VECTORS              : 0x8DFC,
		MAX_COMBINED_TEXTURE_IMAGE_UNITS : 0x8B4D,
		MAX_VERTEX_TEXTURE_IMAGE_UNITS   : 0x8B4C,
		MAX_TEXTURE_IMAGE_UNITS          : 0x8872,
		MAX_FRAGMENT_UNIFORM_VECTORS     : 0x8DFD,
		SHADER_TYPE                      : 0x8B4F,
		DELETE_STATUS                    : 0x8B80,
		LINK_STATUS                      : 0x8B82,
		VALIDATE_STATUS                  : 0x8B83,
		ATTACHED_SHADERS                 : 0x8B85,
		ACTIVE_UNIFORMS                  : 0x8B86,
		ACTIVE_UNIFORM_MAX_LENGTH        : 0x8B87,
		ACTIVE_ATTRIBUTES                : 0x8B89,
		ACTIVE_ATTRIBUTE_MAX_LENGTH      : 0x8B8A,
		SHADING_LANGUAGE_VERSION         : 0x8B8C,
		CURRENT_PROGRAM                  : 0x8B8D,
		
		/* StencilFunction */
		NEVER                          : 0x0200,
		LESS                           : 0x0201,
		EQUAL                          : 0x0202,
		LEQUAL                         : 0x0203,
		GREATER                        : 0x0204,
		NOTEQUAL                       : 0x0205,
		GEQUAL                         : 0x0206,
		ALWAYS                         : 0x0207,
		
		/* StencilOp */
		/*      ZERO */
		KEEP                           : 0x1E00,
		REPLACE                        : 0x1E01,
		INCR                           : 0x1E02,
		DECR                           : 0x1E03,
		INVERT                         : 0x150A,
		INCR_WRAP                      : 0x8507,
		DECR_WRAP                      : 0x8508,
		
		/* StringName */
		VENDOR                         : 0x1F00,
		RENDERER                       : 0x1F01,
		VERSION                        : 0x1F02,
		
		/* TextureMagFilter */
		NEAREST                        : 0x2600,
		LINEAR                         : 0x2601,
		
		/* TextureMinFilter */
		/*      NEAREST */
		/*      LINEAR */
		NEAREST_MIPMAP_NEAREST         : 0x2700,
		LINEAR_MIPMAP_NEAREST          : 0x2701,
		NEAREST_MIPMAP_LINEAR          : 0x2702,
		LINEAR_MIPMAP_LINEAR           : 0x2703,
		
		/* TextureParameterName */
		TEXTURE_MAG_FILTER             : 0x2800,
		TEXTURE_MIN_FILTER             : 0x2801,
		TEXTURE_WRAP_S                 : 0x2802,
		TEXTURE_WRAP_T                 : 0x2803,
		
		/* TextureTarget */
		TEXTURE_2D                     : 0x0DE1,
		TEXTURE                        : 0x1702,
		
		TEXTURE_CUBE_MAP               : 0x8513,
		TEXTURE_BINDING_CUBE_MAP       : 0x8514,
		TEXTURE_CUBE_MAP_POSITIVE_X    : 0x8515,
		TEXTURE_CUBE_MAP_NEGATIVE_X    : 0x8516,
		TEXTURE_CUBE_MAP_POSITIVE_Y    : 0x8517,
		TEXTURE_CUBE_MAP_NEGATIVE_Y    : 0x8518,
		TEXTURE_CUBE_MAP_POSITIVE_Z    : 0x8519,
		TEXTURE_CUBE_MAP_NEGATIVE_Z    : 0x851A,
		MAX_CUBE_MAP_TEXTURE_SIZE      : 0x851C,
		
		/* TextureUnit */
		TEXTURE0                       : 0x84C0,
		TEXTURE1                       : 0x84C1,
		TEXTURE2                       : 0x84C2,
		TEXTURE3                       : 0x84C3,
		TEXTURE4                       : 0x84C4,
		TEXTURE5                       : 0x84C5,
		TEXTURE6                       : 0x84C6,
		TEXTURE7                       : 0x84C7,
		TEXTURE8                       : 0x84C8,
		TEXTURE9                       : 0x84C9,
		TEXTURE10                      : 0x84CA,
		TEXTURE11                      : 0x84CB,
		TEXTURE12                      : 0x84CC,
		TEXTURE13                      : 0x84CD,
		TEXTURE14                      : 0x84CE,
		TEXTURE15                      : 0x84CF,
		TEXTURE16                      : 0x84D0,
		TEXTURE17                      : 0x84D1,
		TEXTURE18                      : 0x84D2,
		TEXTURE19                      : 0x84D3,
		TEXTURE20                      : 0x84D4,
		TEXTURE21                      : 0x84D5,
		TEXTURE22                      : 0x84D6,
		TEXTURE23                      : 0x84D7,
		TEXTURE24                      : 0x84D8,
		TEXTURE25                      : 0x84D9,
		TEXTURE26                      : 0x84DA,
		TEXTURE27                      : 0x84DB,
		TEXTURE28                      : 0x84DC,
		TEXTURE29                      : 0x84DD,
		TEXTURE30                      : 0x84DE,
		TEXTURE31                      : 0x84DF,
		ACTIVE_TEXTURE                 : 0x84E0,
		
		/* TextureWrapMode */
		REPEAT                         : 0x2901,
		CLAMP_TO_EDGE                  : 0x812F,
		MIRRORED_REPEAT                : 0x8370,
		
		/* Uniform Types */
		FLOAT_VEC2                     : 0x8B50,
		FLOAT_VEC3                     : 0x8B51,
		FLOAT_VEC4                     : 0x8B52,
		INT_VEC2                       : 0x8B53,
		INT_VEC3                       : 0x8B54,
		INT_VEC4                       : 0x8B55,
		BOOL                           : 0x8B56,
		BOOL_VEC2                      : 0x8B57,
		BOOL_VEC3                      : 0x8B58,
		BOOL_VEC4                      : 0x8B59,
		FLOAT_MAT2                     : 0x8B5A,
		FLOAT_MAT3                     : 0x8B5B,
		FLOAT_MAT4                     : 0x8B5C,
		SAMPLER_2D                     : 0x8B5E,
		SAMPLER_CUBE                   : 0x8B60,
		
		/* Vertex Arrays */
		VERTEX_ATTRIB_ARRAY_ENABLED        : 0x8622,
		VERTEX_ATTRIB_ARRAY_SIZE           : 0x8623,
		VERTEX_ATTRIB_ARRAY_STRIDE         : 0x8624,
		VERTEX_ATTRIB_ARRAY_TYPE           : 0x8625,
		VERTEX_ATTRIB_ARRAY_NORMALIZED     : 0x886A,
		VERTEX_ATTRIB_ARRAY_POINTER        : 0x8645,
		VERTEX_ATTRIB_ARRAY_BUFFER_BINDING : 0x889F,
		
		/* Read Format */
		IMPLEMENTATION_COLOR_READ_TYPE   : 0x8B9A,
		IMPLEMENTATION_COLOR_READ_FORMAT : 0x8B9B,
		
		/* Shader Source */
		COMPILE_STATUS                 : 0x8B81,
		INFO_LOG_LENGTH                : 0x8B84,
		SHADER_SOURCE_LENGTH           : 0x8B88,
		
		/* Shader Precision-Specified Types */
		LOW_FLOAT                      : 0x8DF0,
		MEDIUM_FLOAT                   : 0x8DF1,
		HIGH_FLOAT                     : 0x8DF2,
		LOW_INT                        : 0x8DF3,
		MEDIUM_INT                     : 0x8DF4,
		HIGH_INT                       : 0x8DF5,
		
		/* Framebuffer Object. */
		FRAMEBUFFER                    : 0x8D40,
		RENDERBUFFER                   : 0x8D41,
		
		RGBA4                          : 0x8056,
		RGB5_A1                        : 0x8057,
		RGB565                         : 0x8D62,
		DEPTH_COMPONENT16              : 0x81A5,
		STENCIL_INDEX                  : 0x1901,
		STENCIL_INDEX8                 : 0x8D48,
		DEPTH_STENCIL                  : 0x84F9,
		
		RENDERBUFFER_WIDTH             : 0x8D42,
		RENDERBUFFER_HEIGHT            : 0x8D43,
		RENDERBUFFER_INTERNAL_FORMAT   : 0x8D44,
		RENDERBUFFER_RED_SIZE          : 0x8D50,
		RENDERBUFFER_GREEN_SIZE        : 0x8D51,
		RENDERBUFFER_BLUE_SIZE         : 0x8D52,
		RENDERBUFFER_ALPHA_SIZE        : 0x8D53,
		RENDERBUFFER_DEPTH_SIZE        : 0x8D54,
		RENDERBUFFER_STENCIL_SIZE      : 0x8D55,
		
		FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE           : 0x8CD0,
		FRAMEBUFFER_ATTACHMENT_OBJECT_NAME           : 0x8CD1,
		FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL         : 0x8CD2,
		FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE : 0x8CD3,
		
		COLOR_ATTACHMENT0              : 0x8CE0,
		DEPTH_ATTACHMENT               : 0x8D00,
		STENCIL_ATTACHMENT             : 0x8D20,
		DEPTH_STENCIL_ATTACHMENT       : 0x821A,
		
		NONE                           : 0,
		
		FRAMEBUFFER_COMPLETE                      : 0x8CD5,
		FRAMEBUFFER_INCOMPLETE_ATTACHMENT         : 0x8CD6,
		FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT : 0x8CD7,
		FRAMEBUFFER_INCOMPLETE_DIMENSIONS         : 0x8CD9,
		FRAMEBUFFER_UNSUPPORTED                   : 0x8CDD,
		
		FRAMEBUFFER_BINDING            : 0x8CA6,
		RENDERBUFFER_BINDING           : 0x8CA7,
		MAX_RENDERBUFFER_SIZE          : 0x84E8,
		
		INVALID_FRAMEBUFFER_OPERATION  : 0x0506,
		
		/* WebGL-specific enums */
		UNPACK_FLIP_Y_WEBGL            : 0x9240,
		UNPACK_PREMULTIPLY_ALPHA_WEBGL : 0x9241,
		CONTEXT_LOST_WEBGL             : 0x9242
	});

	//public

	cWebGLRenderingContext.cWebGLRenderingContext = function(canvas, config, driver) {
		this.canvas = canvas;
		this.driver = driver;
		this.attr = config;
		this._context = cnvgl.createContext(driver);
		this.checkReady();
	};

	cWebGLRenderingContext.activeTexture = function(texture) {
		cnvgl.setContext(this._context);
		cnvgl.activeTexture(texture);
	};

	cWebGLRenderingContext.attachShader = function(program, shader) {
		cnvgl.setContext(this._context);
		cnvgl.attachShader(program.object(), shader.object());
	};

	cWebGLRenderingContext.bindAttribLocation = function(program, index, name) {
		cnvgl.setContext(this._context);
		cnvgl.bindAttribLocation(program.object(), index, name);
	};

	cWebGLRenderingContext.bindBuffer = function(target, buffer) {
		
		if (buffer && buffer.target && buffer.target != target) {
			this.errors.push(this.INVALID_OPERATION);
			return;
		}
		if (target != this.ARRAY_BUFFER && target != this.ELEMENT_ARRAY_BUFFER) {
			this.errors.push(this.INVALID_OPERATION);
			return;
		}

		cnvgl.setContext(this._context);
		cnvgl.bindBuffer(target, buffer ? buffer.object() : 0);

		if (buffer) {
			buffer.target = target;
		}
	};

	cWebGLRenderingContext.bindFramebuffer = function(target, framebuffer) {		
		var _framebuffer = (framebuffer && framebuffer.object) ? framebuffer.object() : framebuffer;
		cnvgl.setContext(this._context);
		cnvgl.bindFramebuffer(target, _framebuffer);
	};

	cWebGLRenderingContext.bindRenderbuffer = function(target, renderbuffer) {
		var _renderbuffer = (renderbuffer && renderbuffer.object) ? renderbuffer.object() : renderbuffer;	
		cnvgl.setContext(this._context);
		cnvgl.bindRenderbuffer(target, _renderbuffer);
	};

	cWebGLRenderingContext.bindTexture = function(target, texture) {
		var _texture = (texture && texture.object) ? texture.object() : texture;
		cnvgl.setContext(this._context);
		cnvgl.bindTexture(target, _texture);
	};

	cWebGLRenderingContext.blendFunc = function(sfactor, dfactor) {
		cnvgl.setContext(this._context);
		cnvgl.blendFunc(sfactor, dfactor);
	};

	cWebGLRenderingContext.bufferData = function(target, data, usage) {
		var size;
		if (typeof data == 'number') {
			size = parseInt(data);
			data = null;
		}
		if (data && data.length) {
			size = data.byteLength;
		}
		if (!size) {
			this.errors.push(this.INVALID_VALUE);
			return;
		}

		cnvgl.setContext(this._context);
		cnvgl.bufferData(target, size, data, usage);
	};

	cWebGLRenderingContext.bufferSubData = function(target, offset, data) {
		var size;
		size = data.byteLength;
		if (!size) {
			this.errors.push(this.INVALID_VALUE);
			return;
		}
		cnvgl.setContext(this._context);
		cnvgl.bufferSubData(target, offset, size, data);
	};

	cWebGLRenderingContext.clear = function(mask) {
		cnvgl.setContext(this._context);
		cnvgl.clear(mask);
	};

	cWebGLRenderingContext.clearColor = function(red, green, blue, alpha) {
		cnvgl.setContext(this._context);
		cnvgl.clearColor(red, green, blue, alpha);
	};

	cWebGLRenderingContext.clearDepth = function(depth) {
		cnvgl.setContext(this._context);
		cnvgl.clearDepth(depth);
	};

	cWebGLRenderingContext.colorMask = function(red, green, blue, alpha) {
		cnvgl.setContext(this._context);
		cnvgl.colorMask(red, green, blue, alpha);
	};

	cWebGLRenderingContext.cullFace = function(mode) {
		cnvgl.setContext(this._context);
		cnvgl.cullFace(mode);
	};

	cWebGLRenderingContext.compileShader = function(shader) {
		var shaderObj, shaderSrc;

		shaderObj = shader.object();
		shaderSrc = shader.getSource();

		cnvgl.setContext(this._context);
		cnvgl.shaderSource(shaderObj, 1, [shaderSrc], [shaderSrc.length]);
		cnvgl.compileShader(shaderObj);
	};

	cWebGLRenderingContext.createBuffer = function() {
		var o = new cWebGLBuffer(this);
		return o;
	};

	cWebGLRenderingContext.createFramebuffer = function() {
		var o = new cWebGLFramebuffer(this);
		return o;
	};
	
	cWebGLRenderingContext.createProgram = function() {
		var o = new cWebGLProgram(this);
		return o;
	};

	cWebGLRenderingContext.createRenderbuffer = function() {
		var o = new cWebGLRenderbuffer(this);
		return o;
	};

	cWebGLRenderingContext.createShader = function(type) {
		var o = new cWebGLShader(this, type);
		return o;
	};

	cWebGLRenderingContext.createTexture = function() {
		var o = new cWebGLTexture(this);
		return o;
	};
	
	cWebGLRenderingContext.depthFunc = function(func) {
		cnvgl.setContext(this._context);
		cnvgl.depthFunc(func);
	};

	cWebGLRenderingContext.depthMask = function(flag) {
		cnvgl.setContext(this._context);
		cnvgl.depthMask(flag);
	};

	cWebGLRenderingContext.disable = function(cap) {
		cnvgl.setContext(this._context);
		cnvgl.disable(cap);
	};

	cWebGLRenderingContext.disableVertexAttribArray = function(index) {
		cnvgl.setContext(this._context);
		cnvgl.disableVertexAttribArray(index);
	};

	cWebGLRenderingContext.drawArrays = function(mode, first, count) {
		cnvgl.setContext(this._context);
		cnvgl.drawArrays(mode, first, count);	
	};

	cWebGLRenderingContext.drawElements = function(mode, count, type, offset) {
		cnvgl.setContext(this._context);
		cnvgl.drawElements(mode, count, type, offset);
	};

	cWebGLRenderingContext.enable = function(cap) {
		cnvgl.setContext(this._context);
		cnvgl.enable(cap);
	};
	
	cWebGLRenderingContext.enableVertexAttribArray = function(index) {
		cnvgl.setContext(this._context);
		cnvgl.enableVertexAttribArray(index);
	};

	cWebGLRenderingContext.flush = function() {
		cnvgl.setContext(this._context);
		cnvgl.flush();
	};

	cWebGLRenderingContext.framebufferRenderbuffer = function(target, attachment, renderbuffertarget, renderbuffer) {
		cnvgl.setContext(this._context);
		cnvgl.framebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer.object());
	};
	
	cWebGLRenderingContext.framebufferTexture2D = function(target, attachment, textarget, texture, level) {
		cnvgl.setContext(this._context);
		cnvgl.framebufferTexture2D(target, attachment, textarget, texture.object(), level);
	};

	cWebGLRenderingContext.frontFace = function(mode) {
		cnvgl.setContext(this._context);
		cnvgl.frontFace(mode);
	};

	cWebGLRenderingContext.generateMipmap = function(target) {
	};

	cWebGLRenderingContext.getAttribLocation = function(program, name) {
		cnvgl.setContext(this._context);
		return cnvgl.getAttribLocation(program.object(), name);
	};

	cWebGLRenderingContext.getContextAttributes = function() {
		return this.attr;
	};

	cWebGLRenderingContext.getError = function() {
		if (this.errors.length > 0) {
			return this.errors.shift();	
		}
		cnvgl.setContext(this._context);
		return cnvgl.getError();
	};

	cWebGLRenderingContext.getParameter = function(pname) {
		return null;
	};

	cWebGLRenderingContext.getProgramParameter = function(program, pname) {
		var params = [];
		cnvgl.setContext(this._context);
		cnvgl.getProgramiv(program.object(), pname, params);
		return params[0];
	};

	cWebGLRenderingContext.getShaderParameter = function(shader, pname) {
		var params = [];
		cnvgl.setContext(this._context);
		cnvgl.getShaderiv(shader.object(), pname, params);
		return params[0];
	};

	cWebGLRenderingContext.getShaderInfoLog = function(shader) {
		var length = [], infoLog = [];
		cnvgl.setContext(this._context);
		cnvgl.getShaderInfoLog(shader.object(), null, length, infoLog);
		return infoLog[0];
	};

	cWebGLRenderingContext.getUniformLocation = function(program, name) {
		var location;
		cnvgl.setContext(this._context);
		location = cnvgl.getUniformLocation(program.object(), name);
		if (location == -1) {
			return 0;
		}
		return new cWebGLUniformLocation(program, location);
	};

	cWebGLRenderingContext.lineWidth = function(width) {
		cnvgl.setContext(this._context);
	};

	cWebGLRenderingContext.linkProgram = function(program) {
		cnvgl.setContext(this._context);
		cnvgl.linkProgram(program.object());
		program.increaseLinkCount();
	};

	cWebGLRenderingContext.onReady = function(func) {
		this._readyFunc = func;
	};

	cWebGLRenderingContext.pixelStorei = function(pname, param) {
		switch (pname) {
			case this.UNPACK_FLIP_Y_WEBGL:
				this._state.UNPACK_FLIP_Y_WEBGL = pname;
				break;
			default:
				cnvgl.setContext(this._context);
				cnvgl.pixelStorei(pname, param);
		}
	};
	
	cWebGLRenderingContext.renderbufferStorage = function(target, internalformat, width, height) {
		cnvgl.setContext(this._context);
		cnvgl.renderbufferStorage(target, internalformat, width, height);
	};
	
	cWebGLRenderingContext.shaderSource = function(shader, string) {
		shader.setSource(string);
	};

	cWebGLRenderingContext.texImage2D = function(target, level, internalformat, format, type, source) {
		var width, height, border, cnv, ctx, cnv, i, j, id, is, t;

		//todo: check origin-clean flag

		if (source instanceof HTMLImageElement) {
			cnv = document.createElement('canvas');
			cnv.width = source.width;
			cnv.height = source.height;
			ctx = cnv.getContext('2d');
			ctx.drawImage(source, 0, 0, source.width, source.height);
			source = cnv;
		}

		if (source instanceof HTMLCanvasElement) {
			if (!ctx) {
				ctx = source.getContext('2d');
			}
			source = ctx.getImageData(0, 0, source.width, source.height);
		}

		if (source.data) {
			width = source.width;
			height = source.height;
			border = 0;
			source = source.data;
		} else {
			width = format;
			height = type;
			border = source;
			format = arguments[6];
			type = arguments[7];
			source = arguments[8];
			if (!source) {
				source = new Uint8Array(width * height * 4);
			}
		}

		//need to invert data rows
		if (this._state.UNPACK_FLIP_Y_WEBGL) {
			t = new Uint8Array(width * height * 4);
			for (i = 0; i < height; i++) {
				for (j = 0; j < width; j++) {
					is = ((width * i) + j) * 4;
					id = ((width * (height - i - 1)) + j) * 4;
					t[id] = source[is];
					t[id + 1] = source[is + 1];
					t[id + 2] = source[is + 2];
					t[id + 3] = source[is + 3];
				}
			}
			source = t;
		}

		cnvgl.setContext(this._context);
		cnvgl.texImage2D(target, level, internalformat, width, height, border, format, type, source);
	};
	
	cWebGLRenderingContext.texParameteri = function(target, pname, param) {
		cnvgl.setContext(this._context);
		cnvgl.texParameteri(target, pname, param);
	};

	cWebGLRenderingContext.uniform1f = function(location, x) {
		cnvgl.setContext(this._context);
		cnvgl.uniform1f(location.location(), x);
	};

	cWebGLRenderingContext.uniform1i = function(location, x) {
		if (!location) {
			return;
		}
		cnvgl.setContext(this._context);
		cnvgl.uniform1i(location.location(), Math.floor(x));
	};

	cWebGLRenderingContext.uniform2f = function(location, x, y) {
		cnvgl.setContext(this._context);
		cnvgl.uniform1f(location.location(), x);
	};
	
	cWebGLRenderingContext.uniform2i = function(location, x, y) {
		cnvgl.setContext(this._context);
		cnvgl.uniform1i(location.location(), x, y);
	};
	
	cWebGLRenderingContext.uniform2u = function(location, x, y) {
		cnvgl.setContext(this._context);
		cnvgl.uniform1u(location.location(), x);
	};

	cWebGLRenderingContext.uniform3f = function(location, x, y, z) {
		cnvgl.setContext(this._context);
		cnvgl.uniform3f(location.location(), x, y, z);
	};

	cWebGLRenderingContext.uniform3i = function(location, x, y, z) {
		cnvgl.setContext(this._context);
		cnvgl.uniform3f(location.location(), x, y, z);
	};

	cWebGLRenderingContext.uniform3u = function(location, x, y, z) {
		cnvgl.setContext(this._context);
		cnvgl.uniform3f(location.location(), x, y, z);
	};

	cWebGLRenderingContext.uniform3fv = function(location, v) {
		cnvgl.setContext(this._context);
		cnvgl.uniform3fv(location.location(), v.length / 3, v);
	};

	cWebGLRenderingContext.uniformMatrix3fv = function(location, transpose, value) {
		cnvgl.setContext(this._context);
		cnvgl.uniformMatrix3fv(location.location(), value.length / 9, transpose, value);
	};
	
	cWebGLRenderingContext.uniformMatrix4fv = function(location, transpose, value) {
		cnvgl.setContext(this._context);
		cnvgl.uniformMatrix4fv(location.location(), value.length / 16, transpose, value);
	};

	cWebGLRenderingContext.useProgram = function(program) {
		cnvgl.setContext(this._context);
		cnvgl.useProgram(program.object());
	};

	cWebGLRenderingContext.vertexAttribPointer = function(idx, size, type, normalized, stride, offset) {
		cnvgl.setContext(this._context);
		cnvgl.vertexAttribPointer(idx, size, type, normalized, stride, offset);
	};

	cWebGLRenderingContext.viewport = function(x, y, width, height) {
		cnvgl.setContext(this._context);
		cnvgl.viewport(x, y, width, height);
	};

	//private:

	cWebGLRenderingContext.checkReady = function() {
		var This;

		if (this.driver.ready && !this._initialized) {
 			this.initialize();
			this._initialized = true;
		}

		if (this.driver.ready && this._readyFunc) {
			this._readyFunc(this);
		} else {
			This = this;
			setTimeout(function() { This.checkReady(); }, 10);
		}
	};

	cWebGLRenderingContext.initialize = function() {
		cnvgl.setContext(this._context);
		cnvgl.viewport(0, 0, this.canvas.width, this.canvas.height);
	};

	return cWebGLRenderingContext.Constructor;

}());

